/*
 * HTGeodesic.java
 * www.bouthier.net
 *
 * The MIT License :
 * -----------------
 * Copyright (c) 2001 Christophe Bouthier
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

// Cdigo alterado por Silvio Evangelista

//package hypertree;
import java.awt.Graphics;
import java.awt.Color;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

/**
 * The HTDrawNodeComposite class implements the Composite design pattern
 * for HTDrawNode.
 * It represents a HTDrawNode which is not a leaf.
 */
class HTDrawNodeComposite
        extends HTDrawNode  {

    private HTModelNodeComposite node = null; // encapsulated HTModelNode
    private Vector children = null; // children of this node
    private Hashtable geodesics = null; // geodesics linking the children


    /* --- Constructor --- */
    /**
     * Constructor.
     *
     * @param father    the father of this node
     * @param node      the encapsulated HTModelNode
     * @param model     the drawing model
     */
    HTDrawNodeComposite(HTDrawNodeComposite father,
            HTModelNodeComposite node, HTDraw model) {
        super(father, node, model);
        this.node = node;
        this.children = new Vector();
        this.geodesics = new Hashtable();

        HTModelNode childNode = null;
        HTDrawNode child = null;
        HTDrawNode brother = null;
        boolean first = true;
        boolean second = false;

        // ***********************************************
        // Ponto para Ordenao. Poderiamos criar um vetor que seria ordenado.
        // Criar novo vetor

        for (Enumeration e = node.children(); e.hasMoreElements();) {

            childNode = (HTModelNode) e.nextElement();

            if (childNode.isLeaf()) {
                child = new HTDrawNode(this, childNode, model);
            } else {
                child = new HTDrawNodeComposite(this, (HTModelNodeComposite) childNode, model);
            }
            addChild(child);
            if (first) {
                brother = child;
                first = false;
                second = true;
            } else if (second) {
                child.setBrother(brother);
                brother.setBrother(child);
                brother = child;
                second = false;
            } else {
                child.setBrother(brother);
                brother = child;
            }
        }

    }


    /* --- Children --- */
    /**
     * Returns the children of this node, 
     * in an Enumeration.
     *
     * @return    the children of this node
     */
    Enumeration children() {
        return children.elements();
    }

    /** 
     * Adds the HTDrawNode as a children.
     *
     * @param child    the child
     */
    void addChild(HTDrawNode child) {
        children.addElement(child);
        geodesics.put(child, new HTGeodesic(getCoordinates(), child.getCoordinates()));
    }


    /* --- Screen Coordinates --- */
    /**
     * Refresh the screen coordinates of this node
     * and recurse on children.
     *
     * @param sOrigin   the origin of the screen plane
     * @param sMax      the (xMax, yMax) point in the screen plane
     */
    void refreshScreenCoordinates(HTCoordS sOrigin, HTCoordS sMax) {
        super.refreshScreenCoordinates(sOrigin, sMax);
        HTDrawNode child = null;

        for (Enumeration e = children(); e.hasMoreElements();) {
            child = (HTDrawNode) e.nextElement();
            child.refreshScreenCoordinates(sOrigin, sMax);
            HTGeodesic geod = (HTGeodesic) geodesics.get(child);
            if (geod != null) {
                geod.refreshScreenCoordinates(sOrigin, sMax);
            }
			
			// Guardar as coordenadas no HTFileNode
			HTCoordS s = child.getScreenCoordinates();
			child.saveXY(s.x, s.y); //*** 12/09/2008
        }
    }

    /* --- Drawing --- */
    /**
     * Draws the branches from this node to 
     * its children.
     *
     * @param g    the graphic context
     */
     // Silvio 12/09/2008
    void drawBranches(Graphics g) {
        HTDrawNode child = null;

        String res;
        for (Enumeration e = children(); e.hasMoreElements();) {
            child = (HTDrawNode) e.nextElement();
            HTGeodesic geod = (HTGeodesic) geodesics.get(child);
            if (geod != null) {
                // 14/11/2005
                if (child.getIsSearch()) {
                    if (child.getHasArc() || child.getHasData()) {
                        if (child.getSearchType()) {
                            g.setColor(new Color(Color.decode(child.getLinkColor()).getRGB()));
			    
			    if (child.getID().indexOf("_H_")>0) {
				if (!child.getViewLineGraph())
					geod.draw(g, true, fastMode, child.getColor());
			    }
			    else {
				geod.draw(g, true, fastMode, child.getColor());
			    }
			    			                                
                        } else {
                            g.setColor(new Color(Color.decode(child.getSearchColor()).getRGB()));
			    if (child.getID().indexOf("_H_")>0) {
				if (!child.getViewLineGraph())
					geod.draw(g, true, fastMode, child.getColor());
			    }
			    else {
				geod.draw(g, true, fastMode, child.getColor());
			    }                           
                        }
                    } else {
                        if (child.getSearchType()) {
                            // Color(int r, int g, int b, int a)
                            int cr = Color.decode(child.getLinkColor()).getRed();
                            int cg = Color.decode(child.getLinkColor()).getGreen();
                            int cb = Color.decode(child.getLinkColor()).getBlue();
                            g.setColor(new Color(cr, cg, cb, 50));
                        } else {
                            g.setColor(new Color(Color.decode(child.getLinkColor()).getRGB()));
                        }
			if (child.getID().indexOf("_H_")>0) {
				if (!child.getViewLineGraph())
					geod.draw(g, fastMode, child.getColor());
			}
			else {
				geod.draw(g, fastMode, child.getColor());
			}
                    }
                } else {
                    g.setColor(new Color(Color.decode(child.getLinkColor()).getRGB()));
			if (child.getID().indexOf("_H_")>0) {
				if (!child.getViewLineGraph())
					geod.draw(g, fastMode, child.getColor());
			}
			else {
				geod.draw(g, fastMode, child.getColor());
			}
                }
            }
            // *****

            // 13/09/2004
            res = child.getNodeClientData();
            // Bruno
            //if (!res.equals("11"))
            if (res.charAt(1) == '1') {
                child.drawBranches(g);            // END
            }
        }
    }

    /**
     * Draws this node.
     *
     * @param g    the graphic context
     *	 bruno
     */
    void drawNodes(Graphics g, boolean MostrarFiguras) {
	    
        //if (fastMode == false) {
        // 13/09/2004
        String res = getNodeClientData();

        if (res.charAt(0) == '1') // Mostra as figuras
        {
            super.drawNodes(g, true);
        } else {
            super.drawNodes(g, false);	// Esconde as figuras
        //super.drawNodes(g);
        // Bruno
        //if (res.equals("11")) return;
        }
        if (res.charAt(1) == '0') {
            return;
        // END
        }
        HTDrawNode child = null;
        for (Enumeration e = children(); e.hasMoreElements();) {
            child = (HTDrawNode) e.nextElement();
            /* Bruno
             * Indica se os ns filhos devem mostrar as figuras
             */
            if (res.charAt(0) == '1') // Mostra as figuras
            {
                child.drawNodes(g, true);
            } else {
                child.drawNodes(g, false);	// Esconde as figuras
            // Fim bruno
            }
        }
    //}

    }

    /**
     * Draws this node.
     *
     * @param g    the graphic context
     */
    void drawNodes0(Graphics g, boolean MostrarFigura) {

        //if (fastMode == false) {

        String res = getNodeClientData();

        if (res.charAt(0) == '1') // Mostra as figuras
        {
            super.drawNodes0(g, true);
        } else {
            super.drawNodes0(g, false);	// Esconde as figuras
        //super.drawNodes0(g, true);

        // 13/09/2004
        //String res = getNodeClientData();
        // Bruno
        //if (!res.equals("11"))
        }
        if (res.charAt(1) == '0') {
            return;
        // END
        }
        HTDrawNode child = null;
        for (Enumeration e = children(); e.hasMoreElements();) {
            child = (HTDrawNode) e.nextElement();
            if (res.charAt(0) == '1') // Mostra as figuras
            {
                child.drawNodes0(g, true);
            } else {
                child.drawNodes0(g, false);	// Esconde as figuras
            //child.drawNodes0(g, true);		    
            }
        }
    //}

    }

    /**
     * Draws hint of this node.
     *
     * @param g    the graphic context
     */
    void drawHint(Graphics g) {
        //if (fastMode == false) {

        super.drawHint(g);

        // 13/09/2004
        String res = getNodeClientData();
        //if (res.equals("11")) return;
        // Bruno
        if (res.charAt(1) == '0') {
            return;
        // END
        }
        HTDrawNode child = null;
        for (Enumeration e = children(); e.hasMoreElements();) {
            child = (HTDrawNode) e.nextElement();
            child.drawHint(g);
            if (child.getHasHint()) {
                //break;
            }
        }
    //}
    }

    /**
     * Returns the minimal distance between this node
     * and his father, his brother, and his children.
     *
     * @return    the minimal distance
     */
    int getSpace() {
        int space = super.getSpace();

        if (!children.isEmpty()) {
            HTDrawNode child = (HTDrawNode) children.firstElement();
            HTCoordS zC = child.getScreenCoordinates();
            int dC = zs.getDistance(zC);

            if (space == -1) {
                return dC;
            } else {
                return Math.min(space, dC);
            }
        } else {
            return space;
        }
    }


    /* --- Translation --- */
    /**
     * Translates this node by the given vector.
     *
     * @param t    the translation vector
     */
    void translate(HTCoordE t) {
        super.translate(t);

        HTDrawNode child = null;
        for (Enumeration e = children(); e.hasMoreElements();) {
            child = (HTDrawNode) e.nextElement();
            child.translate(t);
            HTGeodesic geod = (HTGeodesic) geodesics.get(child);
            if (geod != null) {
                geod.rebuild();
            }
        }

    }

    /**
     * Transform this node by the given transformation.
     *
     * @param t    the transformation
     */
    void transform(HTTransformation t) {
        super.transform(t);

        HTDrawNode child = null;
        for (Enumeration e = children(); e.hasMoreElements();) {
            child = (HTDrawNode) e.nextElement();
            child.transform(t);
            HTGeodesic geod = (HTGeodesic) geodesics.get(child);
            if (geod != null) {
                geod.rebuild();
            }
        }
    }

    /**
     * Ends the translation.
     */
    void endTranslation() {
        super.endTranslation();

        HTDrawNode child = null;
        for (Enumeration e = children(); e.hasMoreElements();) {
            child = (HTDrawNode) e.nextElement();
            child.endTranslation();
        }
    }

    /**
     * Restores the hyperbolic tree to its origin.
     */
    void restore() {
        super.restore();

        HTDrawNode child = null;
        for (Enumeration e = children(); e.hasMoreElements();) {
            child = (HTDrawNode) e.nextElement();
            child.restore();
            HTGeodesic geod = (HTGeodesic) geodesics.get(child);
            if (geod != null) {
                geod.rebuild();
            }
        }

    }

    /**
     * Sets the fast mode, where nodes are no more drawed.
     *
     * @param mode    setting on or off.
     */
    void fastMode(boolean mode) {
        super.fastMode(mode);

        HTDrawNode child = null;
        for (Enumeration e = children(); e.hasMoreElements();) {
            child = (HTDrawNode) e.nextElement();
            child.fastMode(mode);
        }
    }


    /* --- Node searching --- */
    /**
     * Returns the node (if any) whose screen coordinates' zone
     * contains thoses given in parameters.
     *
     * @param zs    the given screen coordinate
     * @return      the searched HTDrawNode if found;
     *              <CODE>null</CODE> otherwise
     */
    HTDrawNode findNode(HTCoordS zs) {
        HTDrawNode result = super.findNode(zs);
        if (result != null) {
            return result;
        } else {
            HTDrawNode child = null;
            for (Enumeration e = children(); e.hasMoreElements();) {
                child = (HTDrawNode) e.nextElement();
                result = child.findNode(zs);
                if (result != null) {
                    return result;
                }
            }
            return null;
        }
    }

    /**
     * Find the node with id.
     *
     * @param id  String
     */
    HTDrawNode findNode(String idNode) {

        HTDrawNode result = super.findNode(idNode);

        if (result != null) {
            return result;
        } else {
            HTDrawNode child = null;

            for (Enumeration e = children(); e.hasMoreElements();) {
                child = (HTDrawNode) e.nextElement();
                result = child.findNode(idNode);
                if (result != null) {
                    return result;
                }
            }
        }
        return null;

    }

    /* --- ToString --- */
    /**
     * Returns a string representation of the object.
     *
     * @return    a String representation of the object
     */
    public String toString() {
        String result = super.toString();
        HTDrawNode child = null;
        result += "\n\tChildren :";
        for (Enumeration e = children(); e.hasMoreElements();) {
            child = (HTDrawNode) e.nextElement();
            result += "\n\t-> " + child.getName();
        }
        return result;
    }
}

